;==============================================================================
    include \masm32\include\masm32rt.inc
;==============================================================================
    .data
      hInst     HMODULE 0
      hdcBM     HDC     0
      hbmp      HBITMAP 0
      clientW   dd      0
      clientH   dd      0
    .code
;==============================================================================

DlgProc proc hwndDlg:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

    LOCAL hdcClient:HDC
    LOCAL ps:PAINTSTRUCT
    LOCAL rc:RECT

    SWITCH uMsg

      CASE WM_INITDIALOG

        ;-------------------------------------------------------
        ; Get and save the width and height of the client area.
        ;-------------------------------------------------------

        invoke GetClientRect, hwndDlg, addr rc
        mov eax, rc.right
        inc eax
        mov clientW, eax
        mov eax, rc.bottom
        inc eax
        mov clientH, eax

        ;-------------------------------------------------------------
        ; Get a DC for the client area and create a compatible memory
        ; DC and compatible bitmap to use as a drawing surface.
        ;-------------------------------------------------------------

        invoke GetDC, hwndDlg
        mov hdcClient, eax
        invoke CreateCompatibleDC, hdcClient
        mov hdcBM, eax
        invoke CreateCompatibleBitmap, hdcClient, clientW, clientH
        mov hbmp, eax

        ;--------------------------------------------------------
        ; Select the bitmap into the memory DC. We have no need
        ; here to save the previously selected object, returned
        ; by the SelectObject function, because we will delete
        ; the DC when we are finished with it.
        ;--------------------------------------------------------

        invoke SelectObject, hdcBM, hbmp

        ;-------------------------------------------------------------
        ; The bitmap, by default, will be filled with black. If there
        ; were a need to fill the bitmap with some other color, this
        ; could be done with something like the commented code below.
        ;
        ; Note that the use of the bitmap width and height as the
        ; right and bottom coordinates for the RECT structure passed
        ; to the FillRect function is intentional. See the Microsoft
        ; documentation for more information.
        ;-------------------------------------------------------------

        ;invoke SetRect, ADDR rc, 0, 0, clientW, clientH
        ;invoke GetStockObject, WHITE_BRUSH
        ;invoke FillRect, hdcBM, ADDR rc, eax

        ;--------------------------------------------
        ; Create a timer that will fire every 50ms.
        ;--------------------------------------------

        invoke SetTimer, hwndDlg, 1, 50, NULL

       ;-------------------------------------
        ; Release the DC for the clent area.
        ;------------------------------------

        invoke ReleaseDC, NULL, hdcClient

      CASE WM_CTLCOLORDLG

        ;------------------------------------------
        ; Set the client area background to black.
        ;------------------------------------------

        invoke GetStockObject, BLACK_BRUSH
        ret

      CASE WM_TIMER

        ;------------------------------------------------------
        ; Draw, on the bitmap, a randomly sized and positioned
        ; ellipse, in blue with a randomly selected intensity.
        ;------------------------------------------------------

        invoke nrandom, 40h
        add eax, 0c0h
        shl eax, 16

        ;------------------------------------------------------------
        ; The handle for the brush used to fill the ellipse is saved
        ; on the stack so we can delete the brush when we finish
        ; using it. The previously selected object for the bitmap
        ; DC, returned by the SelectObject function, is saved on the
        ; stack so we can select it back into the DC.
        ;------------------------------------------------------------

        invoke CreateSolidBrush, eax
        push eax
        invoke SelectObject, hdcBM, eax
        push eax

        ;---------------------------------------------------------------
        ; The monkey motion in the code below is a crude way to ensure
        ; that the minor axis of the ellipse will be at least 20 pixels
        ; in length.
        ;---------------------------------------------------------------

      @@:
        invoke nrandom, clientW
        mov rc.left, eax
        invoke nrandom, clientW
        mov rc.right, eax
        sub eax, rc.left
        js  @b
        cmp eax, 20
        jb  @b
        invoke nrandom, clientH
        mov rc.top, eax
        invoke nrandom, clientH
        mov rc.bottom, eax
        sub eax, rc.top
        js  @b
        cmp eax, 20
        jb  @b

        invoke Ellipse, hdcBM, rc.left, rc.top, rc.right, rc.bottom

        pop eax
        invoke SelectObject, hdcBM, eax
        pop eax
        invoke DeleteObject, eax

        ;---------------------------------------------------
        ; Force a repaint of the client area so the display
        ; will be updated to match the bitmap.
        ;---------------------------------------------------

        invoke InvalidateRect, hwndDlg, NULL, FALSE

      CASE WM_PAINT

        ;--------------------------------------------------------------
        ; This code displays the bitmap by copying it to the paint DC.
        ;--------------------------------------------------------------

        invoke BeginPaint, hwndDlg, ADDR ps

        invoke CreateCompatibleBitmap, ps.hdc, clientW, clientH
        push eax
        invoke SelectObject, ps.hdc, eax
        push eax

        invoke BitBlt, ps.hdc, 0, 0, clientW, clientH, hdcBM, 0, 0, SRCCOPY

        pop eax
        invoke SelectObject, ps.hdc, eax
        pop eax
        invoke DeleteObject, eax

        invoke EndPaint, hwndDlg, ADDR ps

      CASE WM_COMMAND

        SWITCH wParam

          CASE IDCANCEL

            invoke EndDialog, hwndDlg, 0

        ENDSW

      CASE WM_CLOSE

        invoke EndDialog, hwndDlg, 0

    ENDSW

    xor eax, eax
    ret

DlgProc endp

;==============================================================================
start:
;==============================================================================

    invoke GetModuleHandle, NULL
    mov hInst, eax

    Dialog "Test", \
           "MS Sans Serif",8, \
           WS_OVERLAPPED or WS_SYSMENU or DS_CENTER, \
           0,0,0,200,150,1024

    CallModalDialog hInst,0,DlgProc,NULL

    exit
;==============================================================================
end start